@@ -1,15 +1,15 @@
 """Get information and control a GCE IPX800v5."""
-import asyncio
-import socket
+from asyncio import TimeoutError
+from socket import gaierror
 
-import aiohttp
-import async_timeout
+from aiohttp import ClientError, ClientSession
+from async_timeout import timeout
 
 from .const import *
 from .exceptions import (
-    Ipx800CannotConnectError,
-    Ipx800InvalidAuthError,
-    Ipx800RequestError,
+    IPX800CannotConnectError,
+    IPX800InvalidAuthError,
+    IPX800RequestError,
 )
 
 
@@ -22,7 +22,7 @@ class IPX800:
         api_key: str,
         port: int = 80,
         request_timeout: int = 5,
-        session: aiohttp.client.ClientSession = None,
+        session: ClientSession = None,
     ) -> None:
         """Init a IPX800 V5 API."""
         self.host = host
@@ -32,14 +32,15 @@ class IPX800:
         self._base_api_url = f"http://{host}:{port}/api/"
 
         self._api_version = ""
-        self._ipx_config = {}
-        self._extensions_config = []
+        self._ipx_config = {}  # type: dict
+        self._extensions_config = []  # type: list
+        self._objects_config = []  # type: list
 
         self._session = session
         self._close_session = False
 
         if self._session is None:
-            self._session = aiohttp.ClientSession()
+            self._session = ClientSession()
             self._close_session = True
 
     @property
@@ -53,10 +54,15 @@ class IPX800:
         return self._ipx_config
 
     @property
-    def extensions_config(self) -> dict:
+    def extensions_config(self) -> list:
         """Get the config of connected extensions."""
         return self._extensions_config
 
+    @property
+    def objects_config(self) -> list:
+        """Get the config of connected extensions."""
+        return self._objects_config
+
     async def _request_api(
         self, path, data: dict = None, params: dict = None, method: str = "GET"
     ) -> dict:
@@ -66,8 +72,8 @@ class IPX800:
             params_with_api.update(params)
 
         try:
-            with async_timeout.timeout(self._request_timeout):
-                response = await self._session.request(
+            with timeout(self._request_timeout):
+                response = await self._session.request(  # type: ignore
                     method=method,
                     url=self._base_api_url + path,
                     params=params_with_api,
@@ -75,7 +81,7 @@ class IPX800:
                 )
 
             if response.status == 401:
-                raise Ipx800InvalidAuthError()
+                raise IPX800InvalidAuthError()
 
             if response.status >= 200 and response.status <= 206:
                 content = await response.json()
@@ -83,16 +89,16 @@ class IPX800:
                 return content
 
             content = await response.json()
-            raise Ipx800RequestError(
+            raise IPX800RequestError(
                 "IPX800 API request error, error code", response.status
             )
 
-        except asyncio.TimeoutError as exception:
-            raise Ipx800CannotConnectError(
+        except TimeoutError as exception:
+            raise IPX800CannotConnectError(
                 "Timeout occurred while connecting to IPX800."
             ) from exception
-        except (aiohttp.ClientError, socket.gaierror) as exception:
-            raise Ipx800CannotConnectError(
+        except (ClientError, gaierror) as exception:
+            raise IPX800CannotConnectError(
                 "Error occurred while communicating with the IPX800."
             ) from exception
 
@@ -101,7 +107,7 @@ class IPX800:
         try:
             result = await self._request_api("system/ipx")
             return result.get("errorStatus") == 0
-        except Ipx800CannotConnectError:
+        except IPX800CannotConnectError:
             pass
         return False
 
@@ -111,6 +117,8 @@ class IPX800:
         await self.get_ipx_info()
         await self.get_ipx_config()
         await self.get_extensions_config()
+        await self.get_objects_config()
+        return True
 
     async def get_ipx_info(self) -> dict:
         """Get IPX config."""
@@ -119,7 +127,14 @@ class IPX800:
         # TODO More properties
         return infos
 
-    async def get_ipx_config(self) -> dict:
+    async def global_get(self) -> dict:
+        """Get all values from the IPX800 API."""
+        values = {x["_id"]: x for x in await self._request_api("core/io")}
+        values.update({x["_id"]: x for x in await self._request_api("core/ana")})
+        return values
+
+    # Get configs from PX API
+    async def get_ipx_config(self) -> None:
         """Get IPX config."""
         self._ipx_config = await self._request_api(
             "system/ipx", params={"option": "filter_id"}
@@ -135,94 +150,73 @@ class IPX800:
                 ):
                     extensions_config.append(
                         {
-                            EXT_CONFIG_TYPE: type_extension,
-                            EXT_CONFIG_ID: extension["_id"],
-                            EXT_CONFIG_NAME: extension["name"],
-                            EXT_CONFIG_PARAMS: extension,
+                            API_CONFIG_TYPE: type_extension,
+                            API_CONFIG_ID: extension["_id"],
+                            API_CONFIG_NAME: extension["name"],
+                            API_CONFIG_PARAMS: extension,
                         }
                     )
-            except Ipx800RequestError:
+            except IPX800RequestError:
                 print("Error to get %s extensions" % type_extension)
         self._extensions_config = extensions_config
 
-    async def global_get(self) -> dict:
-        """Get all values from the IPX800 API."""
-        values = await self._request_api("core/io")
-        values.update(await self._request_api("core/ana"))
-        return values
+    async def get_objects_config(self) -> None:
+        """Update the list of configured objects."""
+        objects_config = []
+        for type_object in OBJECTS:
+            try:
+                for extension in await self._request_api(
+                    f"object/{type_object}", params={"option": "filter_id"}
+                ):
+                    objects_config.append(
+                        {
+                            API_CONFIG_TYPE: type_object,
+                            API_CONFIG_ID: extension["_id"],
+                            API_CONFIG_NAME: extension["name"],
+                            API_CONFIG_PARAMS: extension,
+                        }
+                    )
+            except IPX800RequestError:
+                print("Error to get %s object" % type_object)
+        self._objects_config = objects_config
+
+    # Get ext or obj configs
+    def get_ext_config(self, ext_type, ext_number) -> dict:
+        """Return the extension config."""
+        extensions = [
+            x for x in self.extensions_config if x[API_CONFIG_TYPE] == ext_type
+        ]
+        return extensions[ext_number - 1][API_CONFIG_PARAMS]
 
     def get_ext_id(self, ext_type, ext_number) -> str:
         """Return the unique extension id generated by the IPX."""
         extensions = [
-            x for x in self.extensions_config if x["type"] == ext_type]
-        return extensions[ext_number - 1]["id"]
-        # TODO Raise error if extension not existing ? or return empty str
+            x for x in self.extensions_config if x[API_CONFIG_TYPE] == ext_type
+        ]
+        return extensions[ext_number - 1][API_CONFIG_ID]
 
-    def get_output_id(
-        self, ext_type, ext_number, output_type, output_number=None
-    ) -> str:
-        """Return the unique id of output."""
-        id_key = (
-            MAPPING_IO_OUTPUT_ID[ext_type]
-            if output_type == TYPE_IO
-            else MAPPING_ANA_OUTPUT_ID[ext_type]
-        )
-        if ext_type == IPX:
-            return self._ipx_config[id_key][output_number - 1]
-        extensions = [
-            x for x in self.extensions_config if x["type"] == ext_type]
-        if output_number is None:
-            return extensions[ext_number - 1][EXT_CONFIG_PARAMS][id_key]
-        return extensions[ext_number - 1][EXT_CONFIG_PARAMS][id_key][output_number - 1]
+    async def get_ext_states(self, ext_type, ext_id) -> dict:
+        """Return all values of extension."""
+        return await self._request_api(f"core/{ext_type}/{ext_id}")
 
-    def get_input_id(
-        self, ext_type, ext_number, output_type, output_number=None
-    ) -> str:
-        """Return the unique id of output."""
-        id_key = (
-            MAPPING_IO_INPUT_ID[ext_type]
-            if output_type == TYPE_IO
-            else MAPPING_ANA_INPUT_ID[ext_type]
-        )
-        if ext_type == IPX:
-            return self._ipx_config[id_key][output_number - 1]
-        extensions = [
-            x for x in self.extensions_config if x["type"] == ext_type]
-        if output_number is None:
-            return extensions[ext_number - 1][EXT_CONFIG_PARAMS][id_key]
-        return extensions[ext_number - 1][EXT_CONFIG_PARAMS][id_key][output_number - 1]
+    def get_obj_config(self, obj_type, obj_number) -> dict:
+        """Return the extension config."""
+        extensions = [x for x in self.objects_config if x[API_CONFIG_TYPE] == obj_type]
+        return extensions[obj_number - 1][API_CONFIG_PARAMS]
 
-    def get_xthl_ids(
-            self, ext_number) -> list:
-        """Return the unique id of X-THL inputs in temp,hum,lum order."""
-        extensions = [
-            x for x in self.extensions_config if x["type"] == EXT_XTHL]
-        params = extensions[ext_number - 1][EXT_CONFIG_PARAMS]
-        return [params["anaTemp_id"], params["anaHum_id"], params["anaLum_id"]]
+    def get_obj_id(self, obj_type, obj_number) -> str:
+        """Return the unique object id generated by the IPX."""
+        objs = [x for x in self.objects_config if x[API_CONFIG_TYPE] == obj_type]
+        return objs[obj_number - 1][API_CONFIG_ID]
 
-    def get_command_id(
-        self, ext_type, ext_number, command_type, command_number=None
-    ) -> str:
-        """Return the unique id of command."""
-        id_key = (
-            MAPPING_IO_COMMAND_ID[ext_type]
-            if command_type == TYPE_IO
-            else MAPPING_ANA_COMMAND_ID[ext_type]
-        )
-        if ext_type == IPX:
-            return self._ipx_config[id_key][command_number - 1]
-        extensions = [
-            x for x in self.extensions_config if x["type"] == ext_type]
-        if command_number is None:
-            return extensions[ext_number - 1][EXT_CONFIG_PARAMS][id_key]
-        return extensions[ext_number - 1][EXT_CONFIG_PARAMS][id_key][command_number - 1]
+    # Get/Update commands
 
     async def get_io(self, id: int) -> bool:
         """Get IO status on the IPX."""
         response = await self._request_api(f"core/io/{id}")
         return response["on"]
 
-    async def get_ana(self, id: int) -> None:
+    async def get_ana(self, id: int) -> float:
         """Get an Analog status on the IPX."""
         response = await self._request_api(f"core/ana/{id}")
         return response["value"]
@@ -231,7 +225,7 @@ class IPX800:
         """Update an IO on the IPX."""
         await self._request_api(f"core/io/{id}", method="PUT", data={command: value})
 
-    async def update_ana(self, id: int, value: int) -> None:
+    async def update_ana(self, id: int, value: float) -> None:
         """Update an Analog on the IPX."""
         await self._request_api(f"core/ana/{id}", method="PUT", data={"value": value})
 
